/********************************** (C) COPYRIGHT *******************************
 * File Name          : CH59x_clk.h
 * Author             : WCH
 * Version            : V1.2
 * Date               : 2021/11/17
 * Description
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for 
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

#ifndef __CH59x_CLK_H__
#define __CH59x_CLK_H__

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief  ϵͳƵ
 */
typedef enum
{
    CLK_SOURCE_LSI = 0x00,
    CLK_SOURCE_LSE,

    CLK_SOURCE_HSE_16MHz = 0x22,
    CLK_SOURCE_HSE_8MHz = 0x24,
    CLK_SOURCE_HSE_6_4MHz = 0x25,
    CLK_SOURCE_HSE_4MHz = 0x28,

    CLK_SOURCE_PLL_80MHz = 0x46, // untested, max allowed per datasheet; possibly req'd for BLE
    CLK_SOURCE_PLL_60MHz = 0x48,
    CLK_SOURCE_PLL_48MHz = (0x40 | 10),
    CLK_SOURCE_PLL_32MHz = (0x40 | 15),
    CLK_SOURCE_PLL_24MHz = (0x40 | 20),
} SYS_CLKTypeDef;

/**
 * @brief  32Kʱѡ
 */
typedef enum
{
    Clk32K_LSI = 0,
    Clk32K_LSE,

} LClk32KTypeDef;

/**
 * @brief  32Mλ
 */
typedef enum
{
    HSE_RCur_75 = 0,
    HSE_RCur_100,
    HSE_RCur_125,
    HSE_RCur_150

} HSECurrentTypeDef;

/**
 * @brief  32Mڲݵλ
 */
typedef enum
{
    HSECap_10p = 0,
    HSECap_12p,
    HSECap_14p,
    HSECap_16p,
    HSECap_18p,
    HSECap_20p,
    HSECap_22p,
    HSECap_24p

} HSECapTypeDef;

/**
 * @brief  32Kλ
 */
typedef enum
{
    LSE_RCur_70 = 0,
    LSE_RCur_100,
    LSE_RCur_140,
    LSE_RCur_200

} LSECurrentTypeDef;

/**
 * @brief  32Kڲݵλ
 */
typedef enum
{
    LSECap_2p = 0,
    LSECap_13p,
    LSECap_14p,
    LSECap_15p,
    LSECap_16p,
    LSECap_17p,
    LSECap_18p,
    LSECap_19p,
    LSECap_20p,
    LSECap_21p,
    LSECap_22p,
    LSECap_23p,
    LSECap_24p,
    LSECap_25p,
    LSECap_26p,
    LSECap_27p

} LSECapTypeDef;

#define RTC_MAX_COUNT             0xA8C00000

#define MAX_DAY                   0x00004000
#define MAX_2_SEC                 0x0000A8C0
//#define	 MAX_SEC		0x545FFFFF

#define BEGYEAR                   2020
#define IsLeapYear(yr)            (!((yr) % 400) || (((yr) % 100) && !((yr) % 4)))
#define YearLength(yr)            (IsLeapYear(yr) ? 366 : 365)
#define monthLength(lpyr, mon)    (((mon) == 1) ? (28 + (lpyr)) : (((mon) > 6) ? (((mon) & 1) ? 31 : 30) : (((mon) & 1) ? 30 : 31)))

/**
 * @brief  rtc timer mode period define
 */
typedef enum
{
    Period_0_125_S = 0, // 0.125s 
    Period_0_25_S,      // 0.25s 
    Period_0_5_S,       // 0.5s 
    Period_1_S,         // 1s 
    Period_2_S,         // 2s 
    Period_4_S,         // 4s 
    Period_8_S,         // 8s 
    Period_16_S,        // 16s 
} RTC_TMRCycTypeDef;

/**
 * @brief  rtc interrupt event define
 */
typedef enum
{
    RTC_TRIG_EVENT = 0, // RTC ¼
    RTC_TMR_EVENT,      // RTC ڶʱ¼

} RTC_EVENTTypeDef;

/**
 * @brief  rtc interrupt event define
 */
typedef enum
{
    RTC_TRIG_MODE = 0, // RTC ģʽ
    RTC_TMR_MODE,      // RTC ڶʱģʽ

} RTC_MODETypeDef;

typedef enum
{
    /* У׼ԽߣʱԽ */
    Level_32 = 3, // ʱ 1.2ms 1000ppm (32M Ƶ)  1100ppm (60M Ƶ)
    Level_64,     // ʱ 2.2ms 800ppm  (32M Ƶ)  1000ppm (60M Ƶ)
    Level_128,    // ʱ 4.2ms 600ppm  (32M Ƶ)  800ppm  (60M Ƶ)

} Cali_LevelTypeDef;

/**
 * @brief   32K ƵʱԴ
 *
 * @param   hc  - ѡ32Kʹڲⲿ
 */
void LClk32K_Select(LClk32KTypeDef hc);

/**
 * @brief   HSE ƫõ
 *
 * @param   c   - 75%,100%,125%,150%
 */
void HSECFG_Current(HSECurrentTypeDef c);

/**
 * @brief   HSE ص
 *
 * @param   c   - refer to HSECapTypeDef
 */
void HSECFG_Capacitance(HSECapTypeDef c);

/**
 * @brief   LSE ƫõ
 *
 * @param   c   - 70%,100%,140%,200%
 */
void LSECFG_Current(LSECurrentTypeDef c);

/**
 * @brief   LSE ص
 *
 * @param   c   - refer to LSECapTypeDef
 */
void LSECFG_Capacitance(LSECapTypeDef c);

void Calibration_LSI(Cali_LevelTypeDef cali_Lv); /* ƵУ׼ڲ32Kʱ */

/**
 * @brief   RTCʱӳʼǰʱ
 *
 * @param   y       - ꣬MAX_Y = BEGYEAR + 44
 * @param   mon     - £MAX_MON = 12
 * @param   d       - գMAX_D = 31
 * @param   h       - СʱMAX_H = 23
 * @param   m       - ÷ӣMAX_M = 59
 * @param   s       - 룬MAX_S = 59
 */
void RTC_InitTime(uint16_t y, uint16_t mon, uint16_t d, uint16_t h, uint16_t m, uint16_t s);

/**
 * @brief   ȡǰʱ
 *
 * @param   py      - ȡ꣬MAX_Y = BEGYEAR + 44
 * @param   pmon    - ȡ£MAX_MON = 12
 * @param   pd      - ȡգMAX_D = 31
 * @param   ph      - ȡСʱMAX_H = 23
 * @param   pm      - ȡķӣMAX_M = 59
 * @param   ps      - ȡ룬MAX_S = 59
 */
void RTC_GetTime(uint16_t *py, uint16_t *pmon, uint16_t *pd, uint16_t *ph, uint16_t *pm, uint16_t *ps);

/**
 * @brief   LSE/LSIʱӣõǰRTC 
 *
 * @param   cyc     - ڼֵMAX_CYC = 0xA8BFFFFF = 2831155199
 */
void RTC_SetCycle32k(uint32_t cyc);

/**
 * @brief   LSE/LSIʱӣȡǰRTC 
 *
 * @return  ǰMAX_CYC = 0xA8BFFFFF = 2831155199
 */
uint32_t RTC_GetCycle32k(void);

/**
 * @brief   RTCʱģʽãעⶨʱ׼̶Ϊ32768Hz
 *
 * @param   t   - refer to RTC_TMRCycTypeDef
 */
void RTC_TRIGFunCfg(uint32_t cyc);

/**
 * @brief   RTCʱģʽãעⶨʱ׼̶Ϊ32768Hz
 *
 * @param   t   - refer to RTC_TMRCycTypeDef
 */
void RTC_TMRFunCfg(RTC_TMRCycTypeDef t);

/**
 * @brief   RTC ģʽܹر
 *
 * @param   m   - Ҫرյĵǰģʽ
 */
void RTC_TRIGFunDisable(RTC_MODETypeDef m);

/**
 * @brief   ȡRTCжϱ־
 *
 * @param   f   - refer to RTC_EVENTTypeDef
 *
 * @return  жϱ־״̬
 */
uint8_t RTC_GetITFlag(RTC_EVENTTypeDef f);

/**
 * @brief   RTCжϱ־
 *
 * @param   f   - refer to RTC_EVENTTypeDef
 */
void RTC_ClearITFlag(RTC_EVENTTypeDef f);

/**
 * @brief   32K ƵʱӵԴ
 */
void LClk32K_Cfg(LClk32KTypeDef hc, FunctionalState s);


#ifdef __cplusplus
}
#endif

#endif // __CH59x_CLK_H__
